package infoh509.cours6.bcard_jaxb;

import java.io.*;
import java.util.*;

import java.awt.*;
import java.awt.event.*;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.Unmarshaller;
import javax.xml.bind.Marshaller;
import javax.xml.transform.stream.StreamSource;

import org.businesscard.*;

public class BCEdit2 extends Frame implements ActionListener {
	
	private static final long serialVersionUID = -839533609996365616L;
	
	//All declarations needed for the user interface
	Button ok = new Button("ok");
	Button delete = new Button("delete");
	Button clear = new Button("clear");
	Button save = new Button("save");
	Button quit = new Button("quit");
	TextField name = new TextField(20);
	TextField title = new TextField(20);
	TextField email = new TextField(20);
	TextField phone = new TextField(20);
	TextField logo = new TextField(20);
	Panel cardpanel = new Panel(new GridLayout(0,1));
	
	//All declarations for the internal representation
	String cardfile;
	java.util.List<CardType> cardlist;
	CardlistType cl;
	JAXBContext jc;
	ObjectFactory objFactory = new ObjectFactory();
	private int current = -1;	

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		new BCEdit2( args[0] );
	}
	
	public BCEdit2( String cardfile ) {
		super();
	    this.cardfile = cardfile;
	    
	    try {
	      jc = JAXBContext.newInstance("org.businesscard");
	      Unmarshaller u = jc.createUnmarshaller();
	      //The next is different from the book !!!
	      JAXBElement<CardlistType> root = 
	    	  u.unmarshal(new StreamSource(new File(cardfile)),CardlistType.class);
	      cl = root.getValue();	      
	      cardlist = cl.getCard();
	    } catch (Exception e) {
	    	System.err.println(e);
	    	System.exit(-1);
	    }
	    // initialize the user interface
	    setLayout(new BorderLayout());
	    ScrollPane s = new ScrollPane();
	    s.setSize(200,0);
	    s.add(cardpanel);
	    add(s, BorderLayout.WEST);
	    Panel l = new Panel(new GridLayout(5,1));
	    l.add(new Label("Name"));
	    l.add(new Label("Title"));
	    l.add(new Label("Email"));
	    l.add(new Label("Phone"));
	    l.add(new Label("Logo"));
	    add(l, BorderLayout.CENTER);
	    Panel f = new Panel(new GridLayout(5,1));
	    f.add(name);
	    f.add(title);
	    f.add(email);
	    f.add(phone);
	    f.add(logo);
	    add(f, BorderLayout.EAST);
	    Panel p = new Panel();
	    ok.addActionListener(this);
	    p.add(ok);
	    delete.addActionListener(this);
	    p.add(delete);
	    clear.addActionListener(this);
	    p.add(clear);
	    save.addActionListener(this);
	    p.add(save);
	    quit.addActionListener(this);
	    p.add(quit);
	    add(p, BorderLayout.SOUTH);
	    addCards();
	    setVisible(true);
	}
	
	private void addCards() {
	    cardpanel.removeAll();
	    
	    for (int i = 0; i < cardlist.size(); i++) {
	    	Button b = new Button(cardlist.get(i).getName());
	    	b.setActionCommand(String.valueOf(i));
	        b.addActionListener(this);
	        cardpanel.add(b);
	    }
	    this.pack();
	}

	
	public void actionPerformed(ActionEvent e) {
		CardType c = null;
		String command = e.getActionCommand();
		if (command.equals("ok")) {
			c = objFactory.createCardType();
			c.setName( name.getText() );
			c.setTitle( title.getText() );
			c.setEmail( email.getText() );
			c.setPhone( phone.getText() );
			LogoType l = objFactory.createLogoType();
			l.setUri(logo.getText());
			c.setLogo( l );
			if (current == -1)
				cardlist.add(c);
			else
				cardlist.set(current, c);
			addCards();
		} else if (command.equals("delete")) {
			if (current != -1) {
				cardlist.remove(current);
				current = -1;
				addCards();
			}
		} else if (command.equals("clear")) {
			current = -1;
			name.setText("");
			title.setText("");
			email.setText("");
			phone.setText("");
			logo.setText("");
		} else if (command.equals("save")) {
			try {
				Marshaller m = jc.createMarshaller();
				m.marshal(cl, new FileOutputStream(cardfile));				
			} catch (Exception ex) {
				System.err.println(ex);
			}
		} else if (command.equals("quit")) {
			System.exit(0);
		} else {
			current = Integer.parseInt(command);
			c = cardlist.get(current);
			name.setText(c.getName());
			title.setText(c.getTitle());
			email.setText(c.getEmail());
			phone.setText(c.getPhone());
			LogoType lt = c.getLogo();
			if (lt != null)
				logo.setText(lt.getUri());
			else 
				logo.setText("");
		}	
	}
}
